@@ -1,6 +1,6 @@
 ;;;; readline.scm --- support functions for command-line editing
 ;;;;
-;;;; 	Copyright (C) 1997, 1999, 2000 Free Software Foundation, Inc.
+;;;; 	Copyright (C) 1997, 1999, 2000, 2001 Free Software Foundation, Inc.
 ;;;; 
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -21,16 +21,22 @@
 ;;;; Extensions based upon code by
 ;;;; Andrew Archibald <aarchiba@undergrad.math.uwaterloo.ca>.
 
+
+
 (define-module (ice-9 readline)
   :use-module (ice-9 session)
   :use-module (ice-9 regex)
+  :use-module (ice-9 buffered-input)
   :no-backtrace)
 
+
+
 ;;; Dynamically link the glue code for accessing the readline library,
 ;;; but only when it isn't already present.
 
 (if (not (feature? 'readline))
-    (dynamic-call "scm_init_readline" (dynamic-link "libguilereadline.so")))
+    (dynamic-call "scm_init_readline"
+                  (dynamic-link "libguilereadline-v-12")))
 
 (if (not (feature? 'readline))
     (scm-error 'misc-error
@@ -39,9 +45,30 @@
 	       '()
 	       '()))
 
+
+
+;;; Run-time options
+
+(export
+ readline-options
+ readline-enable
+ readline-disable)
+(export-syntax
+ readline-set!)
+
+(define-option-interface
+  (readline-options-interface
+   (readline-options readline-enable readline-disable)
+   (readline-set!)))
+
+
+
 ;;; MDJ 980513 <djurfeldt@nada.kth.se>:
 ;;; There should probably be low-level support instead of this code.
 
+;;; Dirk:FIXME:: If the-readline-port, input-port or output-port are closed,
+;;; guile will enter an endless loop or crash.
+
 (define prompt "")
 (define prompt2 "")
 (define input-port (current-input-port))
@@ -49,42 +76,20 @@
 (define read-hook #f)
 
 (define (make-readline-port)
-  (let ((read-string "")
-	(string-index -1))
-    (letrec ((get-character
-	      (lambda ()
-		(cond 
-		 ((eof-object? read-string)
-		  read-string)
-		 ((>= string-index (string-length read-string))
-		  (begin
-		    (set! string-index -1)
-		    #\nl))
-		 ((= string-index -1)
-		  (begin
-		    (set! read-string
-			  (%readline (if (string? prompt)
-					 prompt
-					 (prompt))
-				     input-port
-				     output-port
-				     read-hook))
-		    (set! string-index 0)
-		    (if (not (eof-object? read-string))
-			(begin
-			  (or (string=? read-string "")
-			      (begin
-				(add-history read-string)
-				(set! prompt prompt2)))
-			  (get-character))
-			read-string)))
-		 (else 
-		  (let ((res (string-ref read-string string-index)))
-		    (set! string-index (+ 1 string-index))
-		    res))))))	      
-      (make-soft-port
-       (vector write-char display #f get-character #f)
-       "rw"))))
+  (make-line-buffered-input-port (lambda (continuation?)
+                                   (let* ((prompt (if continuation?
+                                                      prompt2
+                                                      prompt))
+                                          (str (%readline (if (string? prompt)
+                                                              prompt
+                                                              (prompt))
+                                                          input-port
+                                                          output-port
+                                                          read-hook)))
+                                     (or (eof-object? str)
+                                         (string=? str "")
+                                         (add-history str))
+                                     str))))
 
 ;;; We only create one readline port.  There's no point in having
 ;;; more, since they would all share the tty and history ---
@@ -106,7 +111,9 @@
 	  (begin
 	    (do read-history) 
 	    (set! the-readline-port (make-readline-port))
-	    (add-hook! exit-hook (lambda () (do write-history)))))
+	    (add-hook! exit-hook (lambda () 
+				   (do write-history)
+				   (clear-history)))))
       the-readline-port)))
 
 ;;; The user might try to use readline in his programs.  It then
@@ -140,10 +147,24 @@
       (set! prompt2 (car rest))))
 
 (define-public (set-readline-input-port! p)
-  (set! input-port p))
+  (cond ((or (not (file-port? p)) (not (input-port? p)))
+	 (scm-error 'wrong-type-arg "set-readline-input-port!"
+		    "Not a file input port: ~S" (list p) #f))
+	((port-closed? p)
+	 (scm-error 'misc-error "set-readline-input-port!"
+		    "Port not open: ~S" (list p) #f))
+	(else
+	 (set! input-port p))))
 
 (define-public (set-readline-output-port! p)
-  (set! output-port p))
+  (cond ((or (not (file-port? p)) (not (output-port? p)))
+	 (scm-error 'wrong-type-arg "set-readline-input-port!"
+		    "Not a file output port: ~S" (list p) #f))
+	((port-closed? p)
+	 (scm-error 'misc-error "set-readline-output-port!"
+		    "Port not open: ~S" (list p) #f))
+	(else
+	 (set! output-port p))))
 
 (define-public (set-readline-read-hook! h)
   (set! read-hook h))
@@ -167,17 +188,27 @@
       (set! *readline-completion-function* apropos-completion-function)
       ))
 
+(define-public (with-readline-completion-function completer thunk)
+  "With @var{completer} as readline completion function, call @var{thunk}."
+  (let ((old-completer *readline-completion-function*))
+    (dynamic-wind
+	(lambda ()
+	  (set! *readline-completion-function* completer))
+	thunk
+	(lambda ()
+	  (set! *readline-completion-function* old-completer)))))
+
 (define-public (activate-readline)
   (if (and (isatty? (current-input-port))
-	   (not (and (module-defined? the-root-module
-				      'use-emacs-interface)
-		     use-emacs-interface)))
+	   (not (and (module-defined? the-root-module 'use-emacs-interface)
+		     (module-ref the-root-module 'use-emacs-interface))))
       (let ((read-hook (lambda () (run-hook before-read-hook))))
 	(set-current-input-port (readline-port))
 	(set! repl-reader
 	      (lambda (prompt)
 		(dynamic-wind
 		    (lambda ()
+                      (set-buffered-input-continuation?! (readline-port) #f)
 		      (set-readline-prompt! prompt "... ")
 		      (set-readline-read-hook! read-hook))
 		    (lambda () (read))
