@@ -1,4 +1,4 @@
-/* Copyright (C) 1995, 96, 97, 98, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,2000,2001 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,14 +42,12 @@
 
 
 
-#include <stdio.h>
 #include <signal.h>
 #include "libguile/_scm.h"
 #include "libguile/eval.h"
 #include "libguile/throw.h"
 #include "libguile/root.h"
 #include "libguile/smob.h"
-#include "libguile/gc.h"
 
 #include "libguile/validate.h"
 #include "libguile/async.h"
@@ -109,18 +107,17 @@ static unsigned int scm_desired_switch_rate = 0;
 int scm_asyncs_pending_p = 0;
 #endif
 
-static long tc16_async;
+static scm_t_bits tc16_async;
 
 
 
 /* cmm: this has SCM_ prefix because SCM_MAKE_VALIDATE expects it.
    this is ugly.  */
-#define SCM_ASYNCP(X) (SCM_NIMP(X) && (tc16_async == SCM_GCTYP16 (X)))
-
-#define VALIDATE_ASYNC(pos,a) SCM_MAKE_VALIDATE(pos, a, ASYNCP)
+#define SCM_ASYNCP(X)		SCM_TYP16_PREDICATE (tc16_async, X)
+#define VALIDATE_ASYNC(pos,a)	SCM_MAKE_VALIDATE(pos, a, ASYNCP)
 
 #define ASYNC_GOT_IT(X)        (SCM_CELL_WORD_0 (X) >> 16)
-#define SET_ASYNC_GOT_IT(X, V) (SCM_SET_CELL_WORD_0 (X, (SCM_CELL_WORD_0 (X) & ((1 << 16) - 1)) | ((V) << 16)))
+#define SET_ASYNC_GOT_IT(X, V) (SCM_SET_CELL_WORD_0 ((X), SCM_TYP16 (X) | ((V) << 16)))
 #define ASYNC_THUNK(X)         SCM_CELL_OBJECT_1 (X)
 
 
@@ -131,7 +128,7 @@ scm_asyncs_pending ()
 {
   SCM pos;
   pos = scm_asyncs;
-  while (pos != SCM_EOL)
+  while (!SCM_NULLP (pos))
     {
       SCM a = SCM_CAR (pos);
       if (ASYNC_GOT_IT (a))
@@ -141,14 +138,6 @@ scm_asyncs_pending ()
   return 0;
 }
 
-#if 0
-static SCM
-scm_sys_tick_async_thunk (void)
-{
-  scm_deliver_signal (SCM_TICK_SIGNAL);
-  return SCM_BOOL_F;
-}
-#endif
 
 void
 scm_async_click ()
@@ -227,10 +216,6 @@ scm_async_click ()
 	}
     }
 
-  /*
-     if (owe_tick)
-       scm_async_mark (system_signal_asyncs[SCM_SIG_ORD(SCM_TICK_SIGNAL)]); */
-
   SCM_DEFER_INTS;
   if (scm_tick_rate && scm_switch_rate)
     {
@@ -291,7 +276,7 @@ scm_async_click ()
 
 
 static SCM
-mark_async (SCM obj)
+async_mark (SCM obj)
 {
   return ASYNC_THUNK (obj);
 }
@@ -299,8 +284,8 @@ mark_async (SCM obj)
 
 
 SCM_DEFINE (scm_async, "async", 1, 0, 0,
-           (SCM thunk),
-"")
+	    (SCM thunk),
+	    "Create a new async for the procedure @var{thunk}.")
 #define FUNC_NAME s_scm_async
 {
   SCM_RETURN_NEWSMOB (tc16_async, SCM_UNPACK (thunk));
@@ -309,24 +294,19 @@ SCM_DEFINE (scm_async, "async", 1, 0, 0,
 
 SCM_DEFINE (scm_system_async, "system-async", 1, 0, 0,
             (SCM thunk),
-"")
+	    "Create a new async for the procedure @var{thunk}.  Also\n"
+	    "add it to the system's list of active async objects.")
 #define FUNC_NAME s_scm_system_async
 {
-  SCM it;
-  SCM list;
-
-  it = scm_async (thunk);
-  SCM_NEWCELL (list);
-  SCM_SETCAR (list, it);
-  SCM_SETCDR (list, scm_asyncs);
-  scm_asyncs = list;
+  SCM it = scm_async (thunk);
+  scm_asyncs = scm_cons (it, scm_asyncs);
   return it;
 }
 #undef FUNC_NAME
 
 SCM_DEFINE (scm_async_mark, "async-mark", 1, 0, 0,
             (SCM a),
-"")
+	    "Mark the async @var{a} for future execution.")
 #define FUNC_NAME s_scm_async_mark
 {
   VALIDATE_ASYNC (1, a);
@@ -342,7 +322,7 @@ SCM_DEFINE (scm_async_mark, "async-mark", 1, 0, 0,
 
 SCM_DEFINE (scm_system_async_mark, "system-async-mark", 1, 0, 0,
            (SCM a),
-"")
+	    "Mark the async @var{a} for future execution.")
 #define FUNC_NAME s_scm_system_async_mark
 {
   VALIDATE_ASYNC (1, a);
@@ -361,8 +341,8 @@ SCM_DEFINE (scm_system_async_mark, "system-async-mark", 1, 0, 0,
 
 
 SCM_DEFINE (scm_run_asyncs, "run-asyncs", 1, 0, 0,
-           (SCM list_of_a),
-"")
+	    (SCM list_of_a),
+	    "Execute all thunks from the asyncs of the list @var{list_of_a}.")
 #define FUNC_NAME s_scm_run_asyncs
 {
 #ifdef GUILE_OLD_ASYNC_CLICK
@@ -381,7 +361,7 @@ SCM_DEFINE (scm_run_asyncs, "run-asyncs", 1, 0, 0,
       if (ASYNC_GOT_IT (a))
 	{
 	  SET_ASYNC_GOT_IT (a, 0);
-	  scm_apply (ASYNC_THUNK (a), SCM_EOL, SCM_EOL);
+	  scm_call_0 (ASYNC_THUNK (a));
 	}
       scm_mask_ints = 0;
       list_of_a = SCM_CDR (list_of_a);
@@ -394,8 +374,9 @@ SCM_DEFINE (scm_run_asyncs, "run-asyncs", 1, 0, 0,
 
 
 SCM_DEFINE (scm_noop, "noop", 0, 0, 1,
-           (SCM args),
-"")
+	    (SCM args),
+	    "Do nothing.  When called without arguments, return @code{#f},\n"
+	    "otherwise return the first argument.")
 #define FUNC_NAME s_scm_noop
 {
   SCM_VALIDATE_REST_ARGUMENT (args);
@@ -409,8 +390,9 @@ SCM_DEFINE (scm_noop, "noop", 0, 0, 1,
 #ifdef GUILE_OLD_ASYNC_CLICK
 
 SCM_DEFINE (scm_set_tick_rate, "set-tick-rate", 1, 0, 0,
-           (SCM n),
-"")
+	    (SCM n),
+	    "Set the rate of async ticks to @var{n}. Return the old rate\n"
+	    "value.")
 #define FUNC_NAME s_scm_set_tick_rate
 {
   unsigned int old_n = scm_tick_rate;
@@ -426,8 +408,9 @@ SCM_DEFINE (scm_set_tick_rate, "set-tick-rate", 1, 0, 0,
 
 
 SCM_DEFINE (scm_set_switch_rate, "set-switch-rate", 1, 0, 0,
-           (SCM n),
-"")
+	    (SCM n),
+	    "Set the async switch rate to @var{n}. Return the old value\n"
+	    "of the switch rate.")
 #define FUNC_NAME s_scm_set_switch_rate
 {
   unsigned int old_n = scm_switch_rate;
@@ -440,42 +423,12 @@ SCM_DEFINE (scm_set_switch_rate, "set-switch-rate", 1, 0, 0,
 #undef FUNC_NAME
 
 #endif
-
-
-/* points to the GC system-async, so that scm_gc_end can find it.  */
-SCM scm_gc_async;
-
-/* the vcell for gc-thunk.  */
-static SCM scm_gc_vcell;
-
-/* the thunk installed in the GC system-async, which is marked at the
-   end of garbage collection.  */
-static SCM
-scm_sys_gc_async_thunk (void)
-{
-  scm_c_run_hook (scm_after_gc_hook, SCM_EOL);
-
-#if (SCM_DEBUG_DEPRECATED == 0)
-
-  /* The following code will be removed in Guile 1.5.  */
-  if (SCM_NFALSEP (scm_gc_vcell))
-    {
-      SCM proc = SCM_CDR (scm_gc_vcell);
-
-      if (SCM_NFALSEP (proc) && !SCM_UNBNDP (proc))
-	scm_apply (proc, SCM_EOL, SCM_EOL);
-    }
-
-#endif  /* SCM_DEBUG_DEPRECATED == 0 */
-
-  return SCM_UNSPECIFIED;
-}
 
 
 
 SCM_DEFINE (scm_unmask_signals, "unmask-signals", 0, 0, 0,
-           (),
-"")
+	    (),
+	    "Unmask signals. The returned value is not specified.")
 #define FUNC_NAME s_scm_unmask_signals
 {
   scm_mask_ints = 0;
@@ -485,8 +438,8 @@ SCM_DEFINE (scm_unmask_signals, "unmask-signals", 0, 0, 0,
 
 
 SCM_DEFINE (scm_mask_signals, "mask-signals", 0, 0, 0,
-           (),
-"")
+	    (),
+	    "Mask signals. The returned value is not specified.")
 #define FUNC_NAME s_scm_mask_signals
 {
   scm_mask_ints = 1;
@@ -499,12 +452,9 @@ SCM_DEFINE (scm_mask_signals, "mask-signals", 0, 0, 0,
 void
 scm_init_async ()
 {
-  SCM a_thunk;
-  tc16_async = scm_make_smob_type_mfpe ("async", 0,
-                                           mark_async, NULL, NULL, NULL);
-  scm_gc_vcell = scm_sysintern ("gc-thunk", SCM_BOOL_F);
-  a_thunk = scm_make_gsubr ("%gc-thunk", 0, 0, 0, scm_sys_gc_async_thunk);
-  scm_gc_async = scm_system_async (a_thunk);
+  scm_asyncs = SCM_EOL;
+  tc16_async = scm_make_smob_type ("async", 0);
+  scm_set_smob_mark (tc16_async, async_mark);
 
 #include "libguile/async.x"
 }
