@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995, 1996, 2000, 2002 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,7 +42,6 @@
 
 
 
-#include <stdio.h>
 #include "libguile/_scm.h"
 
 #include "libguile/validate.h"
@@ -52,7 +51,7 @@
 
 SCM_DEFINE (scm_not, "not", 1, 0, 0, 
             (SCM x),
-            "Return #t iff X is #f, else return #f.\n")
+            "Return @code{#t} iff @var{x} is @code{#f}, else return @code{#f}.")
 #define FUNC_NAME s_scm_not
 {
   return SCM_BOOL(SCM_FALSEP(x));
@@ -62,7 +61,7 @@ SCM_DEFINE (scm_not, "not", 1, 0, 0,
 
 SCM_DEFINE (scm_boolean_p, "boolean?", 1, 0, 0, 
            (SCM obj),
-            "Return #t iff OBJ is either #t or #f.\n")
+            "Return @code{#t} iff @var{obj} is either @code{#t} or @code{#f}.")
 #define FUNC_NAME s_scm_boolean_p
 {
   return SCM_BOOL (SCM_BOOLP (obj));
