@@ -1,6 +1,6 @@
 /* dynl.c - dynamic linking
  *
- * Copyright (C) 1990, 91, 92, 93, 94, 95, 96, 97, 98, 99, 2000, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1990, 91, 92, 93, 94, 95, 96, 97, 98, 99, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,42 +63,52 @@ maybe_drag_in_eprintf ()
 #endif
 
 #include <stdio.h>
+#include <string.h>
+
 #include "libguile/_scm.h"
 #include "libguile/dynl.h"
 #include "libguile/smob.h"
 #include "libguile/keywords.h"
 #include "libguile/ports.h"
 #include "libguile/strings.h"
-
+#include "libguile/deprecation.h"
 #include "libguile/validate.h"
 
+/* Create a new C argv array from a scheme list of strings. */
+/* Dirk:FIXME:: A quite similar function is implemented in posix.c */
+/* Dirk:FIXME:: In case of assertion errors, we get memory leaks */
+
 /* Converting a list of SCM strings into a argv-style array.  You must
    have ints disabled for the whole lifetime of the created argv (from
    before MAKE_ARGV_FROM_STRINGLIST until after
    MUST_FREE_ARGV). Atleast this is was the documentation for
    MAKARGVFROMSTRS says, it isn't really used that way.
 
-   This code probably belongs into strings.c */
+   This code probably belongs into strings.c 
+   (Dirk: IMO strings.c is not the right place.) */
 
 static char **
 scm_make_argv_from_stringlist (SCM args,int *argcp,const char *subr,int argn)
 {
   char **argv;
-  int argc, i;
+  int argc;
+  int i;
 
   argc = scm_ilength (args);
-  argv = (char **) scm_must_malloc ((1L + argc) * sizeof (char *), subr);
-  for (i = 0; SCM_NNULLP (args); args = SCM_CDR (args), i++) {
+  SCM_ASSERT (argc >= 0, args, argn, subr);
+  argv = (char **) scm_must_malloc ((argc + 1) * sizeof (char *), subr);
+  for (i = 0; !SCM_NULLP (args); args = SCM_CDR (args), ++i) {
+    SCM arg = SCM_CAR (args);
     size_t len;
-    char *dst, *src;
-    SCM str = SCM_CAR (args);
+    char *dst;
+    char *src;
 
-    SCM_ASSERT (SCM_ROSTRINGP (str), str, argn, subr);
-    len = 1 + SCM_ROLENGTH (str);
-    dst = (char *) scm_must_malloc ((long) len, subr);
-    src = SCM_ROCHARS (str);
-    while (len--)
-      dst[len] = src[len];
+    SCM_ASSERT (SCM_STRINGP (arg), args, argn, subr);
+    len = SCM_STRING_LENGTH (arg);
+    src = SCM_STRING_CHARS (arg);
+    dst = (char *) scm_must_malloc (len + 1, subr);
+    memcpy (dst, src, len);
+    dst[len] = 0;
     argv[i] = dst;
   }
 
@@ -117,17 +127,7 @@ scm_must_free_argv(char **argv)
   free (argv);
 }
 
-/* Coerce an arbitrary readonly-string into a zero-terminated string.
- */
-
-static SCM
-scm_coerce_rostring (SCM rostr,const char *subr,int argn)
-{
-  SCM_ASSERT (SCM_ROSTRINGP (rostr), rostr, argn, subr);
-  if (SCM_SUBSTRP (rostr))
-    rostr = scm_makfromstr (SCM_ROCHARS (rostr), SCM_ROLENGTH (rostr), 0);
-  return rostr;
-}
+#if SCM_DEBUG_DEPRECATED == 0
 
 /* Module registry
  */
@@ -150,6 +150,9 @@ scm_register_module_xxx (char *module_name, void *init_func)
 {
   struct moddata *md;
 
+  scm_c_issue_deprecation_warning 
+    ("`scm_register_module_xxx' is deprecated.  Use extensions instead.");
+
   /* XXX - should we (and can we) DEFER_INTS here? */
 
   for (md = registered_mods; md; md = md->link)
@@ -220,6 +223,8 @@ SCM_DEFINE (scm_clear_registered_modules, "c-clear-registered-modules", 0, 0, 0,
 }
 #undef FUNC_NAME
 
+#endif /* !SCM_DEBUG_DEPRECATED */
+
 /* Dispatch to the system dependent files
  *
  * They define some static functions.  These functions are called with
@@ -241,8 +246,13 @@ sysdep_dynl_link (const char *fname, const char *subr)
   handle = lt_dlopenext (fname);
   if (NULL == handle)
     {
+      SCM fn;
+      SCM msg;
+
       SCM_ALLOW_INTS;
-      scm_misc_error (subr, (char *) lt_dlerror (), SCM_EOL);
+      fn = scm_makfrom0str (fname);
+      msg = scm_makfrom0str (lt_dlerror ());
+      scm_misc_error (subr, "file: ~S, message: ~S", scm_list_2 (fn, msg));
     }
   return (void *) handle;
 }
@@ -318,7 +328,7 @@ sysdep_dynl_func (const char *symbol,
 
 #endif
 
-int scm_tc16_dynamic_obj;
+scm_t_bits scm_tc16_dynamic_obj;
 
 #define DYNL_FILENAME(x)        (SCM_CELL_OBJECT_1 (x))
 #define DYNL_HANDLE(x)          ((void *) SCM_CELL_WORD_2 (x))
@@ -326,13 +336,14 @@ int scm_tc16_dynamic_obj;
 
 
 static SCM
-mark_dynl_obj (SCM ptr)
+dynl_obj_mark (SCM ptr)
 {
   return DYNL_FILENAME (ptr);
 }
 
+
 static int
-print_dynl_obj (SCM exp,SCM port,scm_print_state *pstate)
+dynl_obj_print (SCM exp, SCM port, scm_print_state *pstate)
 {
   scm_puts ("#<dynamic-object ", port);
   scm_iprin1 (DYNL_FILENAME (exp), port, pstate);
@@ -344,17 +355,19 @@ print_dynl_obj (SCM exp,SCM port,scm_print_state *pstate)
 
 
 SCM_DEFINE (scm_dynamic_link, "dynamic-link", 1, 0, 0, 
-            (SCM fname),
-	    "Open the dynamic library @var{library-file}.  A library handle\n"
-	    "representing the opened library is returned; this handle should be used\n"
-	    "as the @var{lib} argument to the following functions.")
+            (SCM filename),
+	    "Open the dynamic library called @var{filename}.  A library\n"
+	    "handle representing the opened library is returned; this handle\n"
+	    "should be used as the @var{dobj} argument to the following\n"
+	    "functions.")
 #define FUNC_NAME s_scm_dynamic_link
 {
   void *handle;
 
-  SCM_COERCE_ROSTRING (1, fname);
-  handle = sysdep_dynl_link (SCM_CHARS (fname), FUNC_NAME);
-  SCM_RETURN_NEWSMOB2 (scm_tc16_dynamic_obj, SCM_UNPACK (fname), handle);
+  SCM_VALIDATE_STRING (1, filename);
+  SCM_STRING_COERCE_0TERMINATION_X (filename);
+  handle = sysdep_dynl_link (SCM_STRING_CHARS (filename), FUNC_NAME);
+  SCM_RETURN_NEWSMOB2 (scm_tc16_dynamic_obj, SCM_UNPACK (filename), handle);
 }
 #undef FUNC_NAME
 
@@ -365,20 +378,17 @@ SCM_DEFINE (scm_dynamic_object_p, "dynamic-object?", 1, 0, 0,
 	    "otherwise.")
 #define FUNC_NAME s_scm_dynamic_object_p
 {
-  return SCM_BOOL (SCM_SMOB_PREDICATE (scm_tc16_dynamic_obj, obj));
+  return SCM_BOOL (SCM_TYP16_PREDICATE (scm_tc16_dynamic_obj, obj));
 }
 #undef FUNC_NAME
 
 
 SCM_DEFINE (scm_dynamic_unlink, "dynamic-unlink", 1, 0, 0, 
             (SCM dobj),
-	    "Unlink the library represented by @var{library-handle},\n"
-	    "and remove any imported symbols from the address space.\n"
-	    "GJB:FIXME:DOC: 2nd version below:\n"
 	    "Unlink the indicated object file from the application.  The\n"
-	    "argument @var{dynobj} must have been obtained by a call to\n"
+	    "argument @var{dobj} must have been obtained by a call to\n"
 	    "@code{dynamic-link}.  After @code{dynamic-unlink} has been\n"
-	    "called on @var{dynobj}, its content is no longer accessible.\n")
+	    "called on @var{dobj}, its content is no longer accessible.")
 #define FUNC_NAME s_scm_dynamic_unlink
 {
   /*fixme* GC-problem */
@@ -397,34 +407,35 @@ SCM_DEFINE (scm_dynamic_unlink, "dynamic-unlink", 1, 0, 0,
 
 
 SCM_DEFINE (scm_dynamic_func, "dynamic-func", 2, 0, 0, 
-            (SCM symb, SCM dobj),
-	    "Import the symbol @var{func} from @var{lib} (a dynamic library handle).\n"
-	    "A @dfn{function handle} representing the imported function is returned.\n"
-	    "GJB:FIXME:DOC: 2nd version below\n"
-	    "Search the C function indicated by @var{function} (a string or symbol)\n"
-	    "in @var{dynobj} and return some Scheme object that can later be used\n"
-	    "with @code{dynamic-call} to actually call this function.  Right now,\n"
-	    "these Scheme objects are formed by casting the address of the function\n"
-	    "to @code{long} and converting this number to its Scheme representation.\n\n"
+            (SCM name, SCM dobj),
+	    "Search the dynamic object @var{dobj} for the C function\n"
+	    "indicated by the string @var{name} and return some Scheme\n"
+	    "handle that can later be used with @code{dynamic-call} to\n"
+	    "actually call the function.\n\n"
 	    "Regardless whether your C compiler prepends an underscore @samp{_} to\n"
 	    "the global names in a program, you should @strong{not} include this\n"
 	    "underscore in @var{function}.  Guile knows whether the underscore is\n"
-	    "needed or not and will add it when necessary.\n\n"
-	    "")
+	    "needed or not and will add it when necessary.")
 #define FUNC_NAME s_scm_dynamic_func
 {
+  /* The returned handle is formed by casting the address of the function to a
+   * long value and converting this to a scheme number
+   */
+
   void (*func) ();
 
-  SCM_COERCE_ROSTRING (1, symb);
+  SCM_VALIDATE_STRING (1, name);
   /*fixme* GC-problem */
   SCM_VALIDATE_SMOB (SCM_ARG2, dobj, dynamic_obj);
   if (DYNL_HANDLE (dobj) == NULL) {
     SCM_MISC_ERROR ("Already unlinked: ~S", dobj);
   } else {
+    char *chars;
+
     SCM_DEFER_INTS;
-    func = (void (*) ()) sysdep_dynl_func (SCM_CHARS (symb),
-					   DYNL_HANDLE (dobj),
-					   FUNC_NAME);
+    SCM_STRING_COERCE_0TERMINATION_X (name);
+    chars = SCM_STRING_CHARS (name);
+    func = (void (*) ()) sysdep_dynl_func (chars, DYNL_HANDLE (dobj), FUNC_NAME);
     SCM_ALLOW_INTS;
     return scm_ulong2num ((unsigned long) func);
   }
@@ -434,28 +445,22 @@ SCM_DEFINE (scm_dynamic_func, "dynamic-func", 2, 0, 0,
 
 SCM_DEFINE (scm_dynamic_call, "dynamic-call", 2, 0, 0, 
             (SCM func, SCM dobj),
-	    "Call @var{lib-thunk}, a procedure of no arguments.  If @var{lib-thunk}\n"
-	    "is a string, it is assumed to be a symbol found in the dynamic library\n"
-	    "@var{lib} and is fetched with @code{dynamic-func}.  Otherwise, it should\n"
-	    "be a function handle returned by a previous call to @code{dynamic-func}.\n"
-	    "The return value is unspecified.\n"
-	    "GJB:FIXME:DOC 2nd version below\n"
-	    "Call the C function indicated by @var{function} and @var{dynobj}.  The\n"
-	    "function is passed no arguments and its return value is ignored.  When\n"
-	    "@var{function} is something returned by @code{dynamic-func}, call that\n"
-	    "function and ignore @var{dynobj}.  When @var{function} is a string (or\n"
-	    "symbol, etc.), look it up in @var{dynobj}; this is equivalent to\n\n"
+	    "Call the C function indicated by @var{func} and @var{dobj}.\n"
+	    "The function is passed no arguments and its return value is\n"
+	    "ignored.  When @var{function} is something returned by\n"
+	    "@code{dynamic-func}, call that function and ignore @var{dobj}.\n"
+	    "When @var{func} is a string , look it up in @var{dynobj}; this\n"
+	    "is equivalent to\n"
 	    "@smallexample\n"
-	    "(dynamic-call (dynamic-func @var{function} @var{dynobj} #f))\n"
+	    "(dynamic-call (dynamic-func @var{func} @var{dobj} #f))\n"
 	    "@end smallexample\n\n"
 	    "Interrupts are deferred while the C function is executing (with\n"
-	    "@code{SCM_DEFER_INTS}/@code{SCM_ALLOW_INTS}).\n"
-	    "")
+	    "@code{SCM_DEFER_INTS}/@code{SCM_ALLOW_INTS}).")
 #define FUNC_NAME s_scm_dynamic_call
 {
   void (*fptr) ();
   
-  if (SCM_ROSTRINGP (func))
+  if (SCM_STRINGP (func))
     func = scm_dynamic_func (func, dobj);
   fptr = (void (*) ()) SCM_NUM2ULONG (1, func);
   SCM_DEFER_INTS;
@@ -467,32 +472,25 @@ SCM_DEFINE (scm_dynamic_call, "dynamic-call", 2, 0, 0,
 
 SCM_DEFINE (scm_dynamic_args_call, "dynamic-args-call", 3, 0, 0, 
             (SCM func, SCM dobj, SCM args),
-	    "Call @var{proc}, a dynamically loaded function, passing it the argument\n"
-	    "list @var{args} (a list of strings).  As with @code{dynamic-call},\n"
-	    "@var{proc} should be either a function handle or a string, in which case\n"
-	    "it is first fetched from @var{lib} with @code{dynamic-func}.\n\n"
-	    "@var{proc} is assumed to return an integer, which is used as the return\n"
-	    "value from @code{dynamic-args-call}.\n\n"
-	    "GJB:FIXME:DOC 2nd version below\n"
-	    "Call the C function indicated by @var{function} and @var{dynobj}, just\n"
-	    "like @code{dynamic-call}, but pass it some arguments and return its\n"
-	    "return value.  The C function is expected to take two arguments and\n"
-	    "return an @code{int}, just like @code{main}:\n\n"
+	    "Call the C function indicated by @var{func} and @var{dobj},\n"
+	    "just like @code{dynamic-call}, but pass it some arguments and\n"
+	    "return its return value.  The C function is expected to take\n"
+	    "two arguments and return an @code{int}, just like @code{main}:\n"
 	    "@smallexample\n"
 	    "int c_func (int argc, char **argv);\n"
 	    "@end smallexample\n\n"
-	    "The parameter @var{args} must be a list of strings and is converted into\n"
-	    "an array of @code{char *}.  The array is passed in @var{argv} and its\n"
-	    "size in @var{argc}.  The return value is converted to a Scheme number\n"
-	    "and returned from the call to @code{dynamic-args-call}.\n\n\n"
-	    "")
+	    "The parameter @var{args} must be a list of strings and is\n"
+	    "converted into an array of @code{char *}.  The array is passed\n"
+	    "in @var{argv} and its size in @var{argc}.  The return value is\n"
+	    "converted to a Scheme number and returned from the call to\n"
+	    "@code{dynamic-args-call}.")
 #define FUNC_NAME s_scm_dynamic_args_call
 {
   int (*fptr) (int argc, char **argv);
   int result, argc;
   char **argv;
 
-  if (SCM_ROSTRINGP (func))
+  if (SCM_STRINGP (func))
     func = scm_dynamic_func (func, dobj);
 
   fptr = (int (*) (int, char **)) SCM_NUM2ULONG (1, func);
@@ -510,8 +508,8 @@ void
 scm_init_dynamic_linking ()
 {
   scm_tc16_dynamic_obj = scm_make_smob_type ("dynamic-object", 0);
-  scm_set_smob_mark (scm_tc16_dynamic_obj, mark_dynl_obj);
-  scm_set_smob_print (scm_tc16_dynamic_obj, print_dynl_obj);
+  scm_set_smob_mark (scm_tc16_dynamic_obj, dynl_obj_mark);
+  scm_set_smob_print (scm_tc16_dynamic_obj, dynl_obj_print);
   sysdep_dynl_init ();
 #include "libguile/dynl.x"
 }
