@@ -1,4 +1,4 @@
-/*	Copyright (C) 1998, 1999, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1998,1999,2000,2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@
 #include "libguile/eval.h"
 #include "libguile/macros.h"
 #include "libguile/modules.h"
-#include "libguile/root.h"
+#include "libguile/fluids.h"
 
 #include "libguile/validate.h"
 #include "libguile/evalext.h"
@@ -54,31 +54,34 @@
 SCM_SYMBOL (scm_sym_setter, "setter");
 
 SCM 
-scm_m_generalized_set_x (SCM xorig, SCM env)
+scm_m_generalized_set_x (SCM xorig, SCM env SCM_UNUSED)
 {
   SCM x = SCM_CDR (xorig);
-  SCM_ASSYNT (2 == scm_ilength (x), xorig, scm_s_expression, scm_s_set_x);
+  SCM_ASSYNT (2 == scm_ilength (x), scm_s_expression, scm_s_set_x);
   if (SCM_SYMBOLP (SCM_CAR (x)))
     return scm_cons (SCM_IM_SET_X, x);
   else if (SCM_CONSP (SCM_CAR (x)))
-    return scm_cons (SCM_LIST2 (scm_sym_setter, SCM_CAAR (x)),
-		     scm_append (SCM_LIST2 (SCM_CDAR (x), SCM_CDR (x))));
-  return scm_wta (xorig, scm_s_variable, scm_s_set_x);
+    return scm_cons (scm_list_2 (scm_sym_setter, SCM_CAAR (x)),
+		     scm_append (scm_list_2 (SCM_CDAR (x), SCM_CDR (x))));
+  else
+    scm_misc_error (scm_s_set_x, scm_s_variable, SCM_EOL);
 }
 
 SCM_DEFINE (scm_definedp, "defined?", 1, 1, 0,
             (SCM sym, SCM env),
-	    "Return @code{#t} if @var{sym} is defined in the top-level environment.")
+	    "Return @code{#t} if @var{sym} is defined in the lexical "
+	    "environment @var{env}.  When @var{env} is not specified, "
+	    "look in the top-level environment as defined by the "
+	    "current module.")
 #define FUNC_NAME s_scm_definedp
 {
-  SCM vcell;
+  SCM var;
 
   SCM_VALIDATE_SYMBOL (1,sym);
 
   if (SCM_UNBNDP (env))
-    vcell = scm_sym2vcell(sym,
-			  SCM_CDR (scm_top_level_lookup_closure_var),
-			  SCM_BOOL_F);
+    var = scm_sym2var (sym, scm_current_module_lookup_closure (),
+			 SCM_BOOL_F);
   else
     {
       SCM frames = env;
@@ -103,12 +106,12 @@ SCM_DEFINE (scm_definedp, "defined?", 1, 1, 0,
 		return SCM_BOOL_T;
 	    }
 	}
-    vcell = scm_sym2vcell (sym,
-			   SCM_NIMP (frames) ? SCM_CAR (frames) : SCM_BOOL_F,
-			   SCM_BOOL_F);
+      var = scm_sym2var (sym,
+			 SCM_NIMP (frames) ? SCM_CAR (frames) : SCM_BOOL_F,
+			 SCM_BOOL_F);
     }
 	      
-  return (SCM_FALSEP (vcell) || SCM_UNBNDP (SCM_CDR (vcell))
+  return (SCM_FALSEP (var) || SCM_UNBNDP (SCM_VARIABLE_REF (var))
 	  ? SCM_BOOL_F
 	  : SCM_BOOL_T);
 }
@@ -122,26 +125,17 @@ scm_m_undefine (SCM x, SCM env)
 {
   SCM arg1 = x;
   x = SCM_CDR (x);
-  SCM_ASSYNT (SCM_TOP_LEVEL (env), arg1, "bad placement ", s_undefine);
+  SCM_ASSYNT (SCM_TOP_LEVEL (env), "bad placement ", s_undefine);
   SCM_ASSYNT (SCM_CONSP (x) && SCM_NULLP (SCM_CDR (x)),
-	      arg1, scm_s_expression, s_undefine);
+	      scm_s_expression, s_undefine);
   x = SCM_CAR (x);
-  SCM_ASSYNT (SCM_SYMBOLP (x), arg1, scm_s_variable, s_undefine);
-  arg1 = scm_sym2vcell (x, scm_env_top_level (env), SCM_BOOL_F);
-  SCM_ASSYNT (SCM_NFALSEP (arg1) && !SCM_UNBNDP (SCM_CDR (arg1)),
-	      x, "variable already unbound ", s_undefine);
-#if 0
-#ifndef SCM_RECKLESS
-  if (SCM_NIMP (SCM_CDR (arg1)) && ((SCM) SCM_SNAME (SCM_CDR (arg1)) == x))
-    scm_warn ("undefining built-in ", SCM_CHARS (x));
-  else
-#endif
-    if (5 <= scm_verbose && SCM_UNDEFINED != SCM_CDR (arg1))
-      scm_warn ("redefining ", SCM_CHARS (x));
-#endif
-  SCM_SETCDR (arg1, SCM_UNDEFINED);
+  SCM_ASSYNT (SCM_SYMBOLP (x), scm_s_variable, s_undefine);
+  arg1 = scm_sym2var (x, scm_env_top_level (env), SCM_BOOL_F);
+  SCM_ASSYNT (SCM_NFALSEP (arg1) && !SCM_UNBNDP (SCM_VARIABLE_REF (arg1)),
+	      "variable already unbound ", s_undefine);
+  SCM_VARIABLE_SET (arg1, SCM_UNDEFINED);
 #ifdef SICP
-  return SCM_CAR (arg1);
+  return x;
 #else
   return SCM_UNSPECIFIED;
 #endif
