@@ -2,7 +2,7 @@
 
 #ifndef FILESYSH
 #define FILESYSH
-/*	Copyright (C) 1995,1997,1998, 1999, 2000, 2002 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1997,1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,14 +45,16 @@
 
 
 
-#include <stdio.h>
 #include "libguile/__scm.h"
 
 
 
-extern long scm_tc16_dir;
+extern scm_t_bits scm_tc16_dir;
+
+#define SCM_DIR_FLAG_OPEN (1L << 16)
+
 #define SCM_DIRP(x) (!SCM_IMP (x) && (SCM_TYP16 (x) == scm_tc16_dir))
-#define SCM_OPDIRP(x) (!SCM_IMP (x) && (SCM_CELL_WORD_0 (x) == (scm_tc16_dir | SCM_OPN)))
+#define SCM_DIR_OPEN_P(x) (SCM_CELL_WORD_0 (x) & SCM_DIR_FLAG_OPEN)
 
 
 
@@ -62,6 +64,7 @@ extern SCM scm_umask (SCM mode);
 extern SCM scm_open_fdes (SCM path, SCM flags, SCM mode);
 extern SCM scm_open (SCM path, SCM flags, SCM mode);
 extern SCM scm_close (SCM fd_or_port);
+extern SCM scm_close_fdes (SCM fd);
 extern SCM scm_stat (SCM object);
 extern SCM scm_link (SCM oldpath, SCM newpath);
 extern SCM scm_rename (SCM oldname, SCM newname);
@@ -87,6 +90,14 @@ extern SCM scm_basename (SCM filename, SCM suffix);
 
 extern void scm_init_filesys (void);
 
+
+
+#if (SCM_DEBUG_DEPRECATED == 0)
+
+#define SCM_OPDIRP(x) (SCM_DIRP (x) && (SCM_DIR_OPEN_P (x)))
+
+#endif  /* SCM_DEBUG_DEPRECATED == 0 */
+
 #endif  /* FILESYSH */
 
 /*
