@@ -3,7 +3,7 @@
 #ifndef FLUIDSH
 #define FLUIDSH
 
-/*	Copyright (C) 1996, 2000, 2002 Free Software Foundation, Inc.
+/*	Copyright (C) 1996, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,7 +71,7 @@
    implement a more lightweight version of fluids on top of this basic
    mechanism. */
 
-extern long scm_tc16_fluid;
+extern scm_t_bits scm_tc16_fluid;
 
 #define SCM_FLUIDP(x)    (!SCM_IMP (x) && (SCM_CELL_TYPE (x) == scm_tc16_fluid))
 #define SCM_FLUID_NUM(x) (SCM_CELL_WORD_1 (x))
@@ -93,8 +93,10 @@ SCM scm_fluid_p (SCM fl);
 SCM scm_fluid_ref (SCM fluid);
 SCM scm_fluid_set_x (SCM fluid, SCM value);
 
-SCM scm_internal_with_fluids (SCM fluids, SCM vals,
-                              SCM (*cproc)(void *), void *cdata);
+SCM scm_c_with_fluids (SCM fluids, SCM vals,
+		       SCM (*cproc)(void *), void *cdata);
+SCM scm_c_with_fluid (SCM fluid, SCM val,
+		      SCM (*cproc)(void *), void *cdata);
 SCM scm_with_fluids (SCM fluids, SCM vals, SCM thunk);
 
 SCM scm_make_initial_fluids (void);
@@ -104,6 +106,14 @@ void scm_swap_fluids_reverse (SCM fluids, SCM vals);
 
 void scm_init_fluids (void);
 
+#if SCM_DEBUG_DEPRECATED == 0
+
+/* Use scm_c_with_fluids instead. */
+SCM scm_internal_with_fluids (SCM fluids, SCM vals,
+                              SCM (*cproc)(void *), void *cdata);
+
+#endif
+
 #endif /* !FLUIDSH */
 
 /*
