@@ -1,4 +1,4 @@
-/*      Copyright (C) 1995,1996,1997,1998, 2000 Free Software Foundation, Inc.
+/*      Copyright (C) 1995,1996,1997,1998, 2000, 2001 Free Software Foundation, Inc.
 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,8 +42,6 @@
 
 /* routines to evaluate Scheme code */
 
-#include <stdio.h>
-
 #include "libguile/gh.h"
 
 typedef SCM (*gh_eval_t) (void *data, SCM jmpbuf);
@@ -52,7 +50,7 @@ typedef SCM (*gh_eval_t) (void *data, SCM jmpbuf);
 SCM
 gh_eval_str (const char *scheme_code)
 {
-  return scm_eval_0str (scheme_code);
+  return scm_c_eval_string (scheme_code);
 }
 
 /* evaluate the file by passing it to the lower level scm_primitive_load() */
@@ -72,11 +70,11 @@ eval_str_wrapper (void *data)
 }
 
 SCM
-gh_eval_str_with_catch (const char *scheme_code, scm_catch_handler_t handler)
+gh_eval_str_with_catch (const char *scheme_code, scm_t_catch_handler handler)
 {
   /* FIXME: not there yet */
-  return gh_catch (SCM_BOOL_T, (scm_catch_body_t) eval_str_wrapper, (void *) scheme_code,
-		   (scm_catch_handler_t) handler, (void *) scheme_code);
+  return gh_catch (SCM_BOOL_T, (scm_t_catch_body) eval_str_wrapper, (void *) scheme_code,
+		   (scm_t_catch_handler) handler, (void *) scheme_code);
 }
 
 SCM
@@ -89,9 +87,9 @@ SCM
 gh_eval_str_with_stack_saving_handler (const char *scheme_code)
 {
   return scm_internal_stack_catch (SCM_BOOL_T,
-				   (scm_catch_body_t) eval_str_wrapper,
+				   (scm_t_catch_body) eval_str_wrapper,
 				   (void *) scheme_code,
-				   (scm_catch_handler_t)
+				   (scm_t_catch_handler)
 				   gh_standard_handler,
 				   (void *) scheme_code);
 }
@@ -106,11 +104,11 @@ eval_file_wrapper (void *data)
 }
 
 SCM
-gh_eval_file_with_catch (const char *scheme_code, scm_catch_handler_t handler)
+gh_eval_file_with_catch (const char *scheme_code, scm_t_catch_handler handler)
 {
   /* FIXME: not there yet */
-  return gh_catch (SCM_BOOL_T, (scm_catch_body_t) eval_file_wrapper,
-		   (void *) scheme_code, (scm_catch_handler_t) handler,
+  return gh_catch (SCM_BOOL_T, (scm_t_catch_body) eval_file_wrapper,
+		   (void *) scheme_code, (scm_t_catch_handler) handler,
 		   (void *) scheme_code);
 }
 
