@@ -1,4 +1,4 @@
-/*	Copyright (C) 1997, 1998, 2000 Free Software Foundation, Inc.
+/*	Copyright (C) 1997, 1998, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,9 +40,14 @@
  * If you do not wish that, delete this exception notice.  */
 
 #include <stdio.h>
+#include <errno.h>
 #include <limits.h>
 #include <string.h>
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include "libguile/_scm.h"
 #include "libguile/async.h"
 
@@ -52,10 +57,6 @@
 
 #include "libguile/coop-threads.h"
 
-#ifdef MISSING_BZERO_DECL
-extern void bzero (void *, size_t);
-#endif
-
 
 
 /* COOP queue macros */
@@ -87,11 +88,7 @@ extern void bzero (void *, size_t);
 #error Could not determine suitable definition for SCM_NLONGBITS
 #endif
 
-#ifdef HAVE_BZERO
-#define FD_ZERO_N(pos, n) bzero ((pos), (n))
-#else
 #define FD_ZERO_N(pos, n) memset ((void *) (pos), 0, (n))
-#endif
 
 typedef unsigned long *ulongptr;
 
@@ -103,21 +100,21 @@ int scm_I_am_dead;
 /* This flag indicates that several threads are waiting on the same
    file descriptor.  When this is the case, the common fd sets are
    updated in a more inefficient way.  */
-int collisionp;
+static int collisionp;
 
 /* These are the common fd sets.  When new select calls are made,
    those sets are merged into these.  */
-int gnfds;
-SELECT_TYPE greadfds;
-SELECT_TYPE gwritefds;
-SELECT_TYPE gexceptfds;
+static int gnfds;
+static SELECT_TYPE greadfds;
+static SELECT_TYPE gwritefds;
+static SELECT_TYPE gexceptfds;
 
 /* These are the result sets.  They are used when we call OS select.
    We couldn't use the common fd sets above, since that would destroy
    them.  */
-SELECT_TYPE rreadfds;
-SELECT_TYPE rwritefds;
-SELECT_TYPE rexceptfds;
+static SELECT_TYPE rreadfds;
+static SELECT_TYPE rwritefds;
+static SELECT_TYPE rexceptfds;
 
 /* Constant timeval struct representing a zero timeout which we use
    when polling.  */
