@@ -2,7 +2,7 @@
 
 #ifndef KEYWORDSH
 #define KEYWORDSH
-/*	Copyright (C) 1995,1996,1999, 2000, 2002 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1999, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,7 +49,8 @@
 
 
 
-extern int scm_tc16_keyword;
+extern scm_t_bits scm_tc16_keyword;
+
 #define SCM_KEYWORDP(X)		(!SCM_IMP (X) && (SCM_CELL_TYPE (X) == scm_tc16_keyword))
 #define SCM_KEYWORDSYM(X)	(SCM_CELL_OBJECT_1 (X))
 
