@@ -1,16 +1,16 @@
 /* "net_db.c" network database support
- * Copyright (C) 1995, 96, 97, 98, 99, 2000, 2002 Free Software Foundation, Inc.
- *
+ * Copyright (C) 1995,1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
+ * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *
+ * 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
@@ -49,7 +49,8 @@
  */
 
 
-#include <stdio.h>
+#include <errno.h>
+
 #include "libguile/_scm.h"
 #include "libguile/feature.h"
 #include "libguile/strings.h"
@@ -63,128 +64,23 @@
 #endif
 
 #include <sys/types.h>
+
+#ifdef HAVE_WINSOCK2_H
+#include <winsock2.h>
+#else
 #include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#endif
 
-/* Some systems do not declare this.  Some systems do declare it, as a
-   macro.  */
-#ifndef h_errno
+#if !defined (HAVE_H_ERRNO) && !defined (__MINGW32__)
+/* h_errno not found in netdb.h, maybe this will help.  */
 extern int h_errno;
 #endif
 
 
 
-#ifndef STDC_HEADERS
-int close ();
-#endif /* STDC_HEADERS */
-
-#ifndef HAVE_INET_ATON
-/* for our definition in inet_aton.c, not usually needed.  */
-extern int inet_aton ();
-#endif
-
-SCM_DEFINE (scm_inet_aton, "inet-aton", 1, 0, 0,
-            (SCM address),
-	    "Converts a string containing an Internet host address in the traditional\n"
-	    "dotted decimal notation into an integer.\n\n"
-	    "@smalllisp\n"
-	    "(inet-aton \"127.0.0.1\") @result{} 2130706433\n\n"
-	    "@end smalllisp")
-#define FUNC_NAME s_scm_inet_aton
-{
-  struct in_addr soka;
-
-  SCM_VALIDATE_ROSTRING (1,address);
-  if (SCM_SUBSTRP (address))
-    address = scm_makfromstr (SCM_ROCHARS (address), SCM_ROLENGTH (address), 0);
-  if (inet_aton (SCM_ROCHARS (address), &soka) == 0)
-    SCM_MISC_ERROR ("bad address", SCM_EOL);
-  return scm_ulong2num (ntohl (soka.s_addr));
-}
-#undef FUNC_NAME
-
-
-SCM_DEFINE (scm_inet_ntoa, "inet-ntoa", 1, 0, 0,
-            (SCM inetid),
-	    "Converts an integer Internet host address into a string with the\n"
-	    "traditional dotted decimal representation.\n\n"
-	    "@smalllisp\n"
-	    "(inet-ntoa 2130706433) @result{} \"127.0.0.1\""
-	    "@end smalllisp")
-#define FUNC_NAME s_scm_inet_ntoa
-{
-  struct in_addr addr;
-  char *s;
-  SCM answer;
-  addr.s_addr = htonl (SCM_NUM2ULONG (1,inetid));
-  s = inet_ntoa (addr);
-  answer = scm_makfromstr (s, strlen (s), 0);
-  return answer;
-}
-#undef FUNC_NAME
-
-#ifdef HAVE_INET_NETOF
-SCM_DEFINE (scm_inet_netof, "inet-netof", 1, 0, 0,
-            (SCM address),
-	    "Returns the network number part of the given integer Internet address.\n\n"
-	    "@smalllisp\n"
-	    "(inet-netof 2130706433) @result{} 127\n"
-	    "@end smalllisp")
-#define FUNC_NAME s_scm_inet_netof
-{
-  struct in_addr addr;
-  addr.s_addr = htonl (SCM_NUM2ULONG (1,address));
-  return scm_ulong2num ((unsigned long) inet_netof (addr));
-}
-#undef FUNC_NAME
-#endif
-
-#ifdef HAVE_INET_LNAOF
-SCM_DEFINE (scm_lnaof, "inet-lnaof", 1, 0, 0,
-            (SCM address),
-	    "Returns the local-address-with-network part of the given Internet\n"
-	    "address.\n\n"
-	    "@smalllisp\n"
-	    "(inet-lnaof 2130706433) @result{} 1\n"
-	    "@end smalllisp")
-#define FUNC_NAME s_scm_lnaof
-{
-  struct in_addr addr;
-  addr.s_addr = htonl (SCM_NUM2ULONG (1,address));
-  return scm_ulong2num ((unsigned long) inet_lnaof (addr));
-}
-#undef FUNC_NAME
-#endif
-
-#ifdef HAVE_INET_MAKEADDR
-SCM_DEFINE (scm_inet_makeaddr, "inet-makeaddr", 2, 0, 0,
-            (SCM net, SCM lna),
-	    "Makes an Internet host address by combining the network number @var{net}\n"
-	    "with the local-address-within-network number @var{lna}.\n\n"
-	    "@smalllisp\n"
-	    "(inet-makeaddr 127 1) @result{} 2130706433\n"
-	    "@end smalllisp")
-#define FUNC_NAME s_scm_inet_makeaddr
-{
-  struct in_addr addr;
-  unsigned long netnum;
-  unsigned long lnanum;
-
-#if 0 /* GJB:FIXME:: */
-  SCM_VALIDATE_INUM_COPY (1,net,netnum);
-  SCM_VALIDATE_INUM_COPY (2,lna,lnanum);
-#else
-  netnum = SCM_NUM2ULONG (1, net);
-  lnanum = SCM_NUM2ULONG (2, lna);
-#endif
-  addr = inet_makeaddr (netnum, lnanum);
-  return scm_ulong2num (ntohl (addr.s_addr));
-}
-#undef FUNC_NAME
-#endif
-
 SCM_SYMBOL (scm_host_not_found_key, "host-not-found");
 SCM_SYMBOL (scm_try_again_key, "try-again");
 SCM_SYMBOL (scm_no_recovery_key, "no-recovery");
@@ -208,15 +104,15 @@ static void scm_resolv_error (const char *subr, SCM bad_value)
 	{
 	case HOST_NOT_FOUND:
 	  key = scm_host_not_found_key;
-	  errmsg = "Unknown host";
+	  errmsg = "Unknown host"; 
 	  break;
-	case TRY_AGAIN:
+	case TRY_AGAIN:	
 	  key = scm_try_again_key;
 	  errmsg = "Host name lookup failure";
 	  break;
 	case NO_RECOVERY:
 	  key = scm_no_recovery_key;
-	  errmsg = "Unknown server error";
+	  errmsg = "Unknown server error"; 
 	  break;
 	case NO_DATA:
 	  key = scm_no_data_key;
@@ -238,10 +134,10 @@ static void scm_resolv_error (const char *subr, SCM bad_value)
    Should use reentrant facilities if available.
  */
 
-SCM_DEFINE (scm_gethost, "gethost", 0, 1, 0,
-            (SCM name),
-	    "@deffnx procedure gethostbyname hostname\n"
-	    "@deffnx procedure gethostbyaddr address\n"
+SCM_DEFINE (scm_gethost, "gethost", 0, 1, 0, 
+            (SCM host),
+	    "@deffnx {Scheme Procedure} gethostbyname hostname\n"
+	    "@deffnx {Scheme Procedure} gethostbyaddr address\n"
 	    "Look up a host by name or address, returning a host object.  The\n"
 	    "@code{gethost} procedure will accept either a string name or an integer\n"
 	    "address; if given no arguments, it behaves like @code{gethostent} (see\n"
@@ -253,14 +149,14 @@ SCM_DEFINE (scm_gethost, "gethost", 0, 1, 0,
 	    "@code{system-error} or @code{misc_error} keys.")
 #define FUNC_NAME s_scm_gethost
 {
-  SCM ans = scm_make_vector (SCM_MAKINUM (5), SCM_UNSPECIFIED);
+  SCM ans = scm_c_make_vector (5, SCM_UNSPECIFIED);
   SCM *ve = SCM_VELTS (ans);
   SCM lst = SCM_EOL;
   struct hostent *entry;
   struct in_addr inad;
   char **argv;
   int i = 0;
-  if (SCM_UNBNDP (name))
+  if (SCM_UNBNDP (host))
     {
 #ifdef HAVE_GETHOSTENT
       entry = gethostent ();
@@ -278,21 +174,20 @@ SCM_DEFINE (scm_gethost, "gethost", 0, 1, 0,
 	  return SCM_BOOL_F;
 	}
     }
-  else if (SCM_ROSTRINGP (name))
+  else if (SCM_STRINGP (host))
     {
-      SCM_COERCE_SUBSTR (name);
-      entry = gethostbyname (SCM_ROCHARS (name));
+      SCM_STRING_COERCE_0TERMINATION_X (host);
+      entry = gethostbyname (SCM_STRING_CHARS (host));
     }
   else
     {
-      inad.s_addr = htonl (SCM_NUM2ULONG (1,name));
+      inad.s_addr = htonl (SCM_NUM2ULONG (1, host));
       entry = gethostbyaddr ((char *) &inad, sizeof (inad), AF_INET);
     }
   if (!entry)
-    scm_resolv_error (FUNC_NAME, name);
-
-  ve[0] = scm_makfromstr (entry->h_name,
-			  (scm_sizet) strlen (entry->h_name), 0);
+    scm_resolv_error (FUNC_NAME, host);
+  
+  ve[0] = scm_mem2string (entry->h_name, strlen (entry->h_name));
   ve[1] = scm_makfromstrs (-1, entry->h_aliases);
   ve[2] = SCM_MAKINUM (entry->h_addrtype + 0L);
   ve[3] = SCM_MAKINUM (entry->h_length + 0L);
@@ -323,10 +218,10 @@ SCM_DEFINE (scm_gethost, "gethost", 0, 1, 0,
    operation?), but it seems to work okay.  We'll see.  */
 
 #if defined(HAVE_GETNETENT) && defined(HAVE_GETNETBYNAME) && defined(HAVE_GETNETBYADDR)
-SCM_DEFINE (scm_getnet, "getnet", 0, 1, 0,
-            (SCM name),
-	    "@deffnx procedure getnetbyname net-name\n"
-	    "@deffnx procedure getnetbyaddr net-number\n"
+SCM_DEFINE (scm_getnet, "getnet", 0, 1, 0, 
+            (SCM net),
+	    "@deffnx {Scheme Procedure} getnetbyname net-name\n"
+	    "@deffnx {Scheme Procedure} getnetbyaddr net-number\n"
 	    "Look up a network by name or net number in the network database.  The\n"
 	    "@var{net-name} argument must be a string, and the @var{net-number}\n"
 	    "argument must be an integer.  @code{getnet} will accept either type of\n"
@@ -338,35 +233,33 @@ SCM_DEFINE (scm_getnet, "getnet", 0, 1, 0,
   SCM *ve;
   struct netent *entry;
 
-  ans = scm_make_vector (SCM_MAKINUM (4), SCM_UNSPECIFIED);
+  ans = scm_c_make_vector (4, SCM_UNSPECIFIED);
   ve = SCM_VELTS (ans);
-  if (SCM_UNBNDP (name))
+  if (SCM_UNBNDP (net))
     {
-      errno = 0;
       entry = getnetent ();
       if (! entry)
 	{
-	  if (errno)
-	    SCM_SYSERROR;
-	  else
-	    return SCM_BOOL_F;
+	  /* There's no good way to tell whether zero means an error
+             or end-of-file, so we always return #f.  See `gethost'
+             for details. */
+	  return SCM_BOOL_F;
 	}
     }
-  else if (SCM_ROSTRINGP (name))
+  else if (SCM_STRINGP (net))
     {
-      SCM_COERCE_SUBSTR (name);
-      entry = getnetbyname (SCM_ROCHARS (name));
+      SCM_STRING_COERCE_0TERMINATION_X (net);
+      entry = getnetbyname (SCM_STRING_CHARS (net));
     }
   else
     {
       unsigned long netnum;
-      netnum = SCM_NUM2ULONG (1, name);
+      netnum = SCM_NUM2ULONG (1, net);
       entry = getnetbyaddr (netnum, AF_INET);
     }
   if (!entry)
-    SCM_SYSERROR_MSG ("no such network ~A",
-		      scm_listify (name, SCM_UNDEFINED), errno);
-  ve[0] = scm_makfromstr (entry->n_name, (scm_sizet) strlen (entry->n_name), 0);
+    SCM_SYSERROR_MSG ("no such network ~A", scm_list_1 (net), errno);
+  ve[0] = scm_mem2string (entry->n_name, strlen (entry->n_name));
   ve[1] = scm_makfromstrs (-1, entry->n_aliases);
   ve[2] = SCM_MAKINUM (entry->n_addrtype + 0L);
   ve[3] = scm_ulong2num (entry->n_net + 0L);
@@ -376,10 +269,10 @@ SCM_DEFINE (scm_getnet, "getnet", 0, 1, 0,
 #endif
 
 #ifdef HAVE_GETPROTOENT
-SCM_DEFINE (scm_getproto, "getproto", 0, 1, 0,
-            (SCM name),
-	    "@deffnx procedure getprotobyname name\n"
-	    "@deffnx procedure getprotobynumber number\n"
+SCM_DEFINE (scm_getproto, "getproto", 0, 1, 0, 
+            (SCM protocol),
+	    "@deffnx {Scheme Procedure} getprotobyname name\n"
+	    "@deffnx {Scheme Procedure} getprotobynumber number\n"
 	    "Look up a network protocol by name or by number.  @code{getprotobyname}\n"
 	    "takes a string argument, and @code{getprotobynumber} takes an integer\n"
 	    "argument.  @code{getproto} will accept either type, behaving like\n"
@@ -390,35 +283,33 @@ SCM_DEFINE (scm_getproto, "getproto", 0, 1, 0,
   SCM *ve;
   struct protoent *entry;
 
-  ans = scm_make_vector (SCM_MAKINUM (3), SCM_UNSPECIFIED);
+  ans = scm_c_make_vector (3, SCM_UNSPECIFIED);
   ve = SCM_VELTS (ans);
-  if (SCM_UNBNDP (name))
+  if (SCM_UNBNDP (protocol))
     {
-      errno = 0;
       entry = getprotoent ();
       if (! entry)
 	{
-	  if (errno)
-	    SCM_SYSERROR;
-	  else
-	    return SCM_BOOL_F;
+	  /* There's no good way to tell whether zero means an error
+             or end-of-file, so we always return #f.  See `gethost'
+             for details. */
+	  return SCM_BOOL_F;
 	}
     }
-  else if (SCM_ROSTRINGP (name))
+  else if (SCM_STRINGP (protocol))
     {
-      SCM_COERCE_SUBSTR (name);
-      entry = getprotobyname (SCM_ROCHARS (name));
+      SCM_STRING_COERCE_0TERMINATION_X (protocol);
+      entry = getprotobyname (SCM_STRING_CHARS (protocol));
     }
   else
     {
       unsigned long protonum;
-      protonum = SCM_NUM2ULONG (1,name);
+      protonum = SCM_NUM2ULONG (1, protocol);
       entry = getprotobynumber (protonum);
     }
   if (!entry)
-    SCM_SYSERROR_MSG ("no such protocol ~A",
-		      scm_listify (name, SCM_UNDEFINED), errno);
-  ve[0] = scm_makfromstr (entry->p_name, (scm_sizet) strlen (entry->p_name), 0);
+    SCM_SYSERROR_MSG ("no such protocol ~A", scm_list_1 (protocol), errno);
+  ve[0] = scm_mem2string (entry->p_name, strlen (entry->p_name));
   ve[1] = scm_makfromstrs (-1, entry->p_aliases);
   ve[2] = SCM_MAKINUM (entry->p_proto + 0L);
   return ans;
@@ -432,20 +323,20 @@ scm_return_entry (struct servent *entry)
   SCM ans;
   SCM *ve;
 
-  ans = scm_make_vector (SCM_MAKINUM (4), SCM_UNSPECIFIED);
+  ans = scm_c_make_vector (4, SCM_UNSPECIFIED);
   ve = SCM_VELTS (ans);
-  ve[0] = scm_makfromstr (entry->s_name, (scm_sizet) strlen (entry->s_name), 0);
+  ve[0] = scm_mem2string (entry->s_name, strlen (entry->s_name));
   ve[1] = scm_makfromstrs (-1, entry->s_aliases);
   ve[2] = SCM_MAKINUM (ntohs (entry->s_port) + 0L);
-  ve[3] = scm_makfromstr (entry->s_proto, (scm_sizet) strlen (entry->s_proto), 0);
+  ve[3] = scm_mem2string (entry->s_proto, strlen (entry->s_proto));
   return ans;
 }
 
 #ifdef HAVE_GETSERVENT
 SCM_DEFINE (scm_getserv, "getserv", 0, 2, 0,
-            (SCM name, SCM proto),
-	    "@deffnx procedure getservbyname name protocol\n"
-	    "@deffnx procedure getservbyport port protocol\n"
+            (SCM name, SCM protocol),
+	    "@deffnx {Scheme Procedure} getservbyname name protocol\n"
+	    "@deffnx {Scheme Procedure} getservbyport port protocol\n"
 	    "Look up a network service by name or by service number, and return a\n"
 	    "network service object.  The @var{protocol} argument specifies the name\n"
 	    "of the desired protocol; if the protocol found in the network service\n"
@@ -458,118 +349,103 @@ SCM_DEFINE (scm_getserv, "getserv", 0, 2, 0,
   struct servent *entry;
   if (SCM_UNBNDP (name))
     {
-      errno = 0;
       entry = getservent ();
       if (!entry)
 	{
-	  if (errno)
-	    SCM_SYSERROR;
-	  else
-	    return SCM_BOOL_F;
+	  /* There's no good way to tell whether zero means an error
+             or end-of-file, so we always return #f.  See `gethost'
+             for details. */
+	  return SCM_BOOL_F;
 	}
       return scm_return_entry (entry);
     }
-  SCM_VALIDATE_ROSTRING (2,proto);
-  SCM_COERCE_SUBSTR (proto);
-  if (SCM_ROSTRINGP (name))
+  SCM_VALIDATE_STRING (2, protocol);
+  SCM_STRING_COERCE_0TERMINATION_X (protocol);
+  if (SCM_STRINGP (name))
     {
-      SCM_COERCE_SUBSTR (name);
-      entry = getservbyname (SCM_ROCHARS (name), SCM_ROCHARS (proto));
+      SCM_STRING_COERCE_0TERMINATION_X (name);
+      entry = getservbyname (SCM_STRING_CHARS (name), SCM_STRING_CHARS (protocol));
     }
   else
     {
       SCM_VALIDATE_INUM (1,name);
-      entry = getservbyport (htons (SCM_INUM (name)), SCM_ROCHARS (proto));
+      entry = getservbyport (htons (SCM_INUM (name)), SCM_STRING_CHARS (protocol));
     }
   if (!entry)
-    SCM_SYSERROR_MSG("no such service ~A",
-                     scm_listify (name, SCM_UNDEFINED), errno);
+    SCM_SYSERROR_MSG("no such service ~A", scm_list_1 (name), errno);
   return scm_return_entry (entry);
 }
 #undef FUNC_NAME
 #endif
 
 #if defined(HAVE_SETHOSTENT) && defined(HAVE_ENDHOSTENT)
-SCM_DEFINE (scm_sethost, "sethost", 0, 1, 0,
-            (SCM arg),
+SCM_DEFINE (scm_sethost, "sethost", 0, 1, 0, 
+            (SCM stayopen),
 	    "If @var{stayopen} is omitted, this is equivalent to @code{endhostent}.\n"
 	    "Otherwise it is equivalent to @code{sethostent stayopen}.")
 #define FUNC_NAME s_scm_sethost
 {
-  if (SCM_UNBNDP (arg))
+  if (SCM_UNBNDP (stayopen))
     endhostent ();
   else
-    sethostent (SCM_NFALSEP (arg));
+    sethostent (!SCM_FALSEP (stayopen));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 #endif
 
-#if defined(HAVE_SETNETENT) && defined(HAVE_ENDNETENT)
-SCM_DEFINE (scm_setnet, "setnet", 0, 1, 0,
-            (SCM arg),
+#if defined(HAVE_SETNETENT) && defined(HAVE_ENDNETENT) 
+SCM_DEFINE (scm_setnet, "setnet", 0, 1, 0, 
+            (SCM stayopen),
 	    "If @var{stayopen} is omitted, this is equivalent to @code{endnetent}.\n"
 	    "Otherwise it is equivalent to @code{setnetent stayopen}.")
 #define FUNC_NAME s_scm_setnet
 {
-  if (SCM_UNBNDP (arg))
+  if (SCM_UNBNDP (stayopen))
     endnetent ();
   else
-    setnetent (SCM_NFALSEP (arg));
+    setnetent (!SCM_FALSEP (stayopen));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 #endif
 
 #if defined(HAVE_SETPROTOENT) && defined(HAVE_ENDPROTOENT)
-SCM_DEFINE (scm_setproto, "setproto", 0, 1, 0,
-            (SCM arg),
+SCM_DEFINE (scm_setproto, "setproto", 0, 1, 0, 
+            (SCM stayopen),
 	    "If @var{stayopen} is omitted, this is equivalent to @code{endprotoent}.\n"
 	    "Otherwise it is equivalent to @code{setprotoent stayopen}.")
 #define FUNC_NAME s_scm_setproto
 {
-  if (SCM_UNBNDP (arg))
+  if (SCM_UNBNDP (stayopen))
     endprotoent ();
   else
-    setprotoent (SCM_NFALSEP (arg));
+    setprotoent (!SCM_FALSEP (stayopen));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 #endif
 
 #if defined(HAVE_SETSERVENT) && defined(HAVE_ENDSERVENT)
-SCM_DEFINE (scm_setserv, "setserv", 0, 1, 0,
-            (SCM arg),
+SCM_DEFINE (scm_setserv, "setserv", 0, 1, 0, 
+            (SCM stayopen),
 	    "If @var{stayopen} is omitted, this is equivalent to @code{endservent}.\n"
 	    "Otherwise it is equivalent to @code{setservent stayopen}.")
 #define FUNC_NAME s_scm_setserv
 {
-  if (SCM_UNBNDP (arg))
+  if (SCM_UNBNDP (stayopen))
     endservent ();
   else
-    setservent (SCM_NFALSEP (arg));
+    setservent (!SCM_FALSEP (stayopen));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 #endif
 
 
-void
+void 
 scm_init_net_db ()
 {
-#ifdef INADDR_ANY
-  scm_sysintern ("INADDR_ANY", scm_ulong2num (INADDR_ANY));
-#endif
-#ifdef INADDR_BROADCAST
-  scm_sysintern ("INADDR_BROADCAST", scm_ulong2num (INADDR_BROADCAST));
-#endif
-#ifdef INADDR_NONE
-  scm_sysintern ("INADDR_NONE", scm_ulong2num (INADDR_NONE));
-#endif
-#ifdef INADDR_LOOPBACK
-  scm_sysintern ("INADDR_LOOPBACK", scm_ulong2num (INADDR_LOOPBACK));
-#endif
-
   scm_add_feature ("net-db");
 #include "libguile/net_db.x"
 }
