@@ -2,7 +2,7 @@
 
 #ifndef NUMBERSH
 #define NUMBERSH
-/*	Copyright (C) 1995, 1996, 1998, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,2000,2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,35 +50,36 @@
 
 
 
+/* Immediate Numbers, also known as fixnums
+ *
+ * Inums are exact integer data that fits within an SCM word.  */
+
+#define SCM_I_FIXNUM_BIT \
+  (SCM_LONG_BIT - 2)
+#define SCM_MOST_POSITIVE_FIXNUM \
+  ((((scm_t_signed_bits) 1) << (SCM_I_FIXNUM_BIT - 1)) - 1)
+#define SCM_MOST_NEGATIVE_FIXNUM \
+  (-((scm_t_signed_bits) SCM_MOST_POSITIVE_FIXNUM) - 1)
 
-/* Immediate Numbers 
- *
- * Inums are exact integer data that fits within an SCM word.
- *
- * SCM_INUMP applies only to values known to be Scheme objects.
- * In particular, SCM_INUMP (SCM_CAR (x)) is valid only if x is known
- * to be a SCM_CONSP.  If x is only known to be a SCM_NIMP, 
- * SCM_INUMP (SCM_CAR (x)) can give wrong answers.
- */
 
 /* SCM_SRS is signed right shift */
 #if (-1 == (((-1) << 2) + 2) >> 2)
 # define SCM_SRS(x, y) ((x) >> (y))
 #else
-# define SCM_SRS(x, y) ((SCM_UNPACK (x) < 0) ? ~((~SCM_UNPACK (x)) >> (y)) : (SCM_UNPACK (x) >> (y)))
+# define SCM_SRS(x, y) ((x) < 0 ? ~((~(x)) >> (y)) : ((x) >> (y)))
 #endif /* (-1 == (((-1) << 2) + 2) >> 2) */
 
 
 #define SCM_INUMP(x)	(2 & SCM_UNPACK (x))
 #define SCM_NINUMP(x) 	(!SCM_INUMP (x))
-#define SCM_MAKINUM(x)  (SCM_PACK (((x) << 2) + 2L))
-#define SCM_INUM(x)     (SCM_SRS (SCM_UNPACK (x), 2))
+#define SCM_MAKINUM(x)  (SCM_PACK ((((scm_t_signed_bits) (x)) << 2) + 2))
+#define SCM_INUM(x)     (SCM_SRS ((scm_t_signed_bits) SCM_UNPACK (x), 2))
 
 
 /* SCM_FIXABLE is true if its long argument can be encoded in an SCM_INUM. */
 #define SCM_POSFIXABLE(n) ((n) <= SCM_MOST_POSITIVE_FIXNUM)
 #define SCM_NEGFIXABLE(n) ((n) >= SCM_MOST_NEGATIVE_FIXNUM)
-#define SCM_FIXABLE(n) (SCM_POSFIXABLE(n) && SCM_NEGFIXABLE(n))
+#define SCM_FIXABLE(n) (SCM_POSFIXABLE (n) && SCM_NEGFIXABLE (n))
 
 
 /* A name for 0. */
@@ -109,15 +110,7 @@
 /* SCM_INTBUFLEN is the maximum number of characters neccessary for the
  * printed or scm_string representation of an exact immediate.
  */
-
-#ifndef SCM_CHAR_BIT
-# define SCM_CHAR_BIT 8
-#endif /* ndef SCM_CHAR_BIT */
-#ifndef SCM_LONG_BIT
-# define SCM_LONG_BIT (SCM_CHAR_BIT*sizeof(long)/sizeof(char))
-#endif /* ndef SCM_LONG_BIT */
-#define SCM_INTBUFLEN (5+SCM_LONG_BIT)
-
+#define SCM_INTBUFLEN (5 + SCM_LONG_BIT)
 
 
 
@@ -131,9 +124,10 @@
 #define SCM_REALP(x) (SCM_NIMP (x) && SCM_TYP16 (x) == scm_tc16_real)
 #define SCM_COMPLEXP(x) (SCM_NIMP (x) && SCM_TYP16 (x) == scm_tc16_complex)
 
-#define SCM_REAL_VALUE(x) (((scm_double_t *) SCM2PTR (x))->real)
-#define SCM_COMPLEX_REAL(x) (((scm_complex_t *) SCM_CELL_WORD_1 (x))->real)
-#define SCM_COMPLEX_IMAG(x) (((scm_complex_t *) SCM_CELL_WORD_1 (x))->imag)
+#define SCM_REAL_VALUE(x) (((scm_t_double *) SCM2PTR (x))->real)
+#define SCM_COMPLEX_MEM(x) ((scm_t_complex *) SCM_CELL_WORD_1 (x))
+#define SCM_COMPLEX_REAL(x) (SCM_COMPLEX_MEM (x)->real)
+#define SCM_COMPLEX_IMAG(x) (SCM_COMPLEX_MEM (x)->imag)
 
 /* Define SCM_BIGDIG to an integer type whose size is smaller than long if
  * you want bignums.  SCM_BIGRAD is one greater than the biggest SCM_BIGDIG. 
@@ -155,9 +149,10 @@
 # endif /* def _UNICOS */
 
 # define SCM_BIGRAD (1L << SCM_BITSPERDIG)
-# define SCM_DIGSPERLONG ((scm_sizet)((sizeof(long)*SCM_CHAR_BIT+SCM_BITSPERDIG-1)/SCM_BITSPERDIG))
-# define SCM_BIGUP(x) ((unsigned long)(x) << SCM_BITSPERDIG)
-# define SCM_LONGLONGBIGUP(x) ((ulong_long)(x) << SCM_BITSPERDIG)
+# define SCM_DIGSPERLONG ((size_t)((sizeof(long)*SCM_CHAR_BIT+SCM_BITSPERDIG-1)/SCM_BITSPERDIG))
+# define SCM_I_BIGUP(type, x) ((type)(x) << SCM_BITSPERDIG)
+# define SCM_BIGUP(x) SCM_I_BIGUP (unsigned long, x)
+# define SCM_LONGLONGBIGUP(x) SCM_I_BIGUP (unsigned long long, x)
 # define SCM_BIGDN(x) ((x) >> SCM_BITSPERDIG)
 # define SCM_BIGLO(x) ((x) & (SCM_BIGRAD-1))
 #endif /* def BIGNUMS */
@@ -176,7 +171,8 @@
 #define SCM_BIGSIZEFIELD 17
 #define SCM_BIGSIGN(x) (SCM_CELL_WORD_0 (x) & SCM_BIGSIGNFLAG)
 #define SCM_BDIGITS(x) ((SCM_BIGDIG *) (SCM_CELL_WORD_1 (x)))
-#define SCM_NUMDIGS(x) ((scm_sizet) (SCM_CELL_WORD_0 (x) >> SCM_BIGSIZEFIELD))
+#define SCM_SET_BIGNUM_BASE(n, b) (SCM_SET_CELL_WORD_1 ((n), (b)))
+#define SCM_NUMDIGS(x) ((size_t) (SCM_CELL_WORD_0 (x) >> SCM_BIGSIZEFIELD))
 #define SCM_SETNUMDIGS(x, v, sign) \
   SCM_SET_CELL_WORD_0 (x, \
 	      scm_tc16_big \
@@ -185,18 +181,18 @@
 
 
 
-typedef struct scm_double_t
+typedef struct scm_t_double
 {
   SCM type;
   SCM pad;
   double real;
-} scm_double_t;
+} scm_t_double;
 
-typedef struct scm_complex_t
+typedef struct scm_t_complex
 {
   double real;
   double imag;
-} scm_complex_t;
+} scm_t_complex;
 
 
 
@@ -220,24 +216,47 @@ extern SCM scm_ash (SCM n, SCM cnt);
 extern SCM scm_bit_extract (SCM n, SCM start, SCM end);
 extern SCM scm_logcount (SCM n);
 extern SCM scm_integer_length (SCM n);
-extern SCM scm_mkbig (scm_sizet nlen, int sign);
-extern SCM scm_big2inum (SCM b, scm_sizet l);
-extern SCM scm_adjbig (SCM b, scm_sizet nlen);
+extern SCM scm_i_mkbig (size_t nlen, int sign);
+extern SCM scm_i_big2inum (SCM b, size_t l);
+extern SCM scm_i_adjbig (SCM b, size_t nlen);
+extern SCM scm_i_normbig (SCM b);
+extern SCM scm_i_copybig (SCM b, int sign);
+extern SCM scm_i_short2big (short n);
+extern SCM scm_i_ushort2big (unsigned short n);
+extern SCM scm_i_int2big (int n);
+extern SCM scm_i_uint2big (unsigned int n);
+extern SCM scm_i_long2big (long n);
+extern SCM scm_i_ulong2big (unsigned long n);
+extern SCM scm_i_size2big (size_t n);
+extern SCM scm_i_ptrdiff2big (ptrdiff_t n);
+
+
+#if (SCM_DEBUG_DEPRECATED == 0)
+extern SCM scm_big2inum (SCM b, size_t l);
+extern SCM scm_mkbig (size_t nlen, int sign);
+extern SCM scm_adjbig (SCM b, size_t len);
 extern SCM scm_normbig (SCM b);
 extern SCM scm_copybig (SCM b, int sign);
-extern SCM scm_long2big (long n);
-#ifdef HAVE_LONG_LONGS
-extern SCM scm_long_long2big (long_long n);
+
+#define SCM_FIXNUM_BIT SCM_I_FIXNUM_BIT
 #endif
+
+#ifdef HAVE_LONG_LONGS
+extern SCM scm_i_long_long2big (long long n);
+extern SCM scm_i_ulong_long2big (unsigned long long n);
+#endif
+
+#if (SCM_DEBUG_DEPRECATED == 0)
 extern SCM scm_2ulong2big (unsigned long * np);
-extern SCM scm_ulong2big (unsigned long n);
+#endif
+
 extern int scm_bigcomp (SCM x, SCM y);
 extern long scm_pseudolong (long x);
 extern void scm_longdigs (long x, SCM_BIGDIG digs[]);
-extern SCM scm_addbig (SCM_BIGDIG *x, scm_sizet nx, int xsgn, SCM bigy, int sgny);
-extern SCM scm_mulbig (SCM_BIGDIG *x, scm_sizet nx, SCM_BIGDIG *y, scm_sizet ny, int sgn);
-extern unsigned int scm_divbigdig (SCM_BIGDIG *ds, scm_sizet h, SCM_BIGDIG div);
-extern scm_sizet scm_iint2str (long num, int rad, char *p);
+extern SCM scm_addbig (SCM_BIGDIG *x, size_t nx, int xsgn, SCM bigy, int sgny);
+extern SCM scm_mulbig (SCM_BIGDIG *x, size_t nx, SCM_BIGDIG *y, size_t ny, int sgn);
+extern unsigned int scm_divbigdig (SCM_BIGDIG *ds, size_t h, SCM_BIGDIG div);
+extern size_t scm_iint2str (long num, int rad, char *p);
 extern SCM scm_number_to_string (SCM x, SCM radix);
 extern int scm_print_real (SCM sexp, SCM port, scm_print_state *pstate);
 extern int scm_print_complex (SCM sexp, SCM port, scm_print_state *pstate);
@@ -286,46 +305,60 @@ extern SCM scm_magnitude (SCM z);
 extern SCM scm_angle (SCM z);
 extern SCM scm_inexact_to_exact (SCM z);
 extern SCM scm_trunc (SCM x);
-extern SCM scm_dbl2big (double d);
-extern double scm_big2dbl (SCM b);
-extern SCM scm_long2num (long sl);
-extern SCM scm_ulong2num (unsigned long sl);
-extern long scm_num2long (SCM num, char *pos, const char *s_caller);
-#ifdef HAVE_LONG_LONGS
-extern SCM scm_long_long2num (long_long sl);
-extern long_long scm_num2long_long (SCM num, char *pos,
-                                    const char *s_caller);
-#endif
-extern unsigned long scm_num2ulong (SCM num, char *pos,
-                                    const char *s_caller);
-extern void scm_init_numbers (void);
-
-
+extern SCM scm_i_dbl2big (double d);
 
 #if (SCM_DEBUG_DEPRECATED == 0)
-
-typedef struct scm_dblproc
-{
-  char *scm_string;
-  double (*cproc) ();
-} scm_dblproc;
-
-#define SCM_UNEGFIXABLE(n) ((n) <= -SCM_MOST_NEGATIVE_FIXNUM)
-#define SCM_FLOBUFLEN (10+2*(sizeof(double)/sizeof(char)*SCM_CHAR_BIT*3+9)/10)
-#define SCM_INEXP(x) SCM_INEXACTP(x)
-#define SCM_CPLXP(x) SCM_COMPLEXP(x)
-#define SCM_REAL(x) (SCM_SLOPPY_REALP (x) ? SCM_REAL_VALUE (x) : SCM_COMPLEX_REAL (x))
-#define SCM_IMAG(x) (SCM_SLOPPY_REALP (x) ? 0.0 : SCM_COMPLEX_IMAG (x))
-#define SCM_REALPART(x) (SCM_SLOPPY_REALP (x) ? SCM_REAL_VALUE (x) : SCM_COMPLEX_REAL (x))
-#define scm_makdbl scm_make_complex
-#define SCM_SINGP(x) 0
-#define SCM_NUM2DBL(x) scm_num2dbl(x, "SCM_NUM2DBL")
-
-#ifndef SCM_BIGDIG
-# define SCM_NO_BIGDIG
+extern SCM scm_dbl2big (double d);
 #endif
 
-#endif  /* SCM_DEBUG_DEPRECATED == 0 */
+extern double scm_i_big2dbl (SCM b);
+
+#if (SCM_DEBUG_DEPRECATED == 0)
+extern double scm_big2dbl (SCM b);
+#endif
+
+extern SCM scm_short2num (short n);
+extern SCM scm_ushort2num (unsigned short n);
+extern SCM scm_int2num (int n);
+extern SCM scm_uint2num (unsigned int n);
+extern SCM scm_long2num (long n);
+extern SCM scm_ulong2num (unsigned long n);
+extern SCM scm_size2num (size_t n);
+extern SCM scm_ptrdiff2num (ptrdiff_t n);
+extern short scm_num2short (SCM num, unsigned long int pos,
+                            const char *s_caller);
+extern unsigned short scm_num2ushort (SCM num, unsigned long int pos,
+                                      const char *s_caller);
+extern int scm_num2int (SCM num, unsigned long int pos,
+                        const char *s_caller);
+extern unsigned int scm_num2uint (SCM num, unsigned long int pos,
+                                  const char *s_caller);
+extern long scm_num2long (SCM num, unsigned long int pos,
+			  const char *s_caller);
+extern unsigned long scm_num2ulong (SCM num, unsigned long int pos,
+                                    const char *s_caller);
+extern ptrdiff_t scm_num2ptrdiff (SCM num, unsigned long int pos,
+                                  const char *s_caller);
+extern size_t scm_num2size (SCM num, unsigned long int pos,
+                            const char *s_caller);
+#ifdef HAVE_LONG_LONGS
+extern SCM scm_long_long2num (long long sl);
+extern SCM scm_ulong_long2num (unsigned long long sl);
+extern long long scm_num2long_long (SCM num, unsigned long int pos,
+                                    const char *s_caller);
+extern unsigned long long scm_num2ulong_long (SCM num, unsigned long int pos,
+				      const char *s_caller);
+#endif
+
+extern float scm_num2float (SCM num, unsigned long int pos,
+                            const char *s_caller);
+extern SCM scm_float2num (float n);
+
+extern double scm_num2double (SCM num, unsigned long int pos,
+                            const char *s_caller);
+extern SCM scm_double2num (double n);
+
+extern void scm_init_numbers (void);
 
 #endif  /* NUMBERSH */
 
