@@ -2,7 +2,7 @@
 
 #ifndef OPTIONSH
 #define OPTIONSH
-/*	Copyright (C) 1995,1996 Free Software Foundation
+/*	Copyright (C) 1995,1996, 2000, 2001 Free Software Foundation
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,21 +51,36 @@
 
 
 
-typedef struct scm_option
+typedef struct scm_t_option
 {
   int type;
   char *name;
+
+  /*
+    schizophrenic use: both SCM and int
+   */
   unsigned long val;
+  /* SCM val */
   char *doc;
-} scm_option;
+} scm_t_option;
+
+#if (SCM_DEBUG_DEPRECATED == 0)
+# define scm_option scm_t_option
+#endif
 
 #define SCM_OPTION_BOOLEAN 0
 #define SCM_OPTION_INTEGER 1
 #define SCM_OPTION_SCM     2
 
 
-extern SCM scm_options SCM_P ((SCM new_mode, scm_option options[], int n, const char *s));
-extern void scm_init_opts SCM_P ((SCM (*func) (SCM), scm_option options[], int n));
-extern void scm_init_options SCM_P ((void));
+extern SCM scm_options (SCM new_mode, scm_t_option options[], int n, const char *s);
+extern void scm_init_opts (SCM (*func) (SCM), scm_t_option options[], int n);
+extern void scm_init_options (void);
 
 #endif /* OPTIONSH */
+
+/*
+  Local Variables:
+  c-file-style: "gnu"
+  End:
+*/
