@@ -2,7 +2,7 @@
 
 #ifndef SCMSIGSH
 #define SCMSIGSH
-/*	Copyright (C) 1995,1996,1997,1998 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997,1998, 2000 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,12 +48,22 @@
 
 
 extern SCM scm_sigaction (SCM signum, SCM handler, SCM flags);
-extern SCM scm_restore_signals SCM_P ((void));
-extern SCM scm_alarm SCM_P ((SCM i));
-extern SCM scm_pause SCM_P ((void));
-extern SCM scm_sleep SCM_P ((SCM i));
-extern SCM scm_usleep SCM_P ((SCM i));
-extern SCM scm_raise SCM_P ((SCM sig));
-extern void scm_init_scmsigs SCM_P ((void));
+extern SCM scm_restore_signals (void);
+extern SCM scm_alarm (SCM i);
+extern SCM scm_setitimer (SCM which_timer,
+                          SCM interval_seconds, SCM interval_microseconds,
+                          SCM value_seconds, SCM value_microseconds);
+extern SCM scm_getitimer (SCM which_timer);
+extern SCM scm_pause (void);
+extern SCM scm_sleep (SCM i);
+extern SCM scm_usleep (SCM i);
+extern SCM scm_raise (SCM sig);
+extern void scm_init_scmsigs (void);
 
 #endif  /* SCMSIGSH */
+
+/*
+  Local Variables:
+  c-file-style: "gnu"
+  End:
+*/
