@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996, 1999, 2000, 2002 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996, 1999, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,9 +42,9 @@
 
 
 
+#include <stdio.h>
 #include "libguile/_scm.h"
 #include "libguile/strings.h"
-#include "libguile/versiondat.h"
 
 #include "libguile/version.h"
 
@@ -54,10 +54,11 @@
 SCM_DEFINE (scm_major_version, "major-version", 0, 0, 0, 
             (),
             "Return a string containing Guile's major version number.\n"
-            "E.g., \"1\".")
+            "E.g., the 1 in \"1.6.5\".")
 #define FUNC_NAME s_scm_major_version
 {
-  return scm_makfrom0str (GUILE_MAJOR_VERSION);
+  return scm_number_to_string (SCM_MAKINUM(SCM_MAJOR_VERSION),
+                               SCM_MAKINUM(10));
 }
 #undef FUNC_NAME
 
@@ -66,10 +67,24 @@ SCM_DEFINE (scm_major_version, "major-version", 0, 0, 0,
 SCM_DEFINE (scm_minor_version, "minor-version", 0, 0, 0, 
             (),
             "Return a string containing Guile's minor version number.\n"
-            "E.g., \"3.5\".")
+            "E.g., the 6 in \"1.6.5\".")
 #define FUNC_NAME s_scm_minor_version
 {
-  return scm_makfrom0str (GUILE_MINOR_VERSION);
+  return scm_number_to_string (SCM_MAKINUM(SCM_MINOR_VERSION),
+                               SCM_MAKINUM(10));
+}
+#undef FUNC_NAME
+
+/* Return a Scheme string containing Guile's micro version number.  */
+
+SCM_DEFINE (scm_micro_version, "micro-version", 0, 0, 0, 
+            (),
+            "Return a string containing Guile's micro version number.\n"
+            "E.g., the 5 in \"1.6.5\".")
+#define FUNC_NAME s_scm_micro_version
+{
+  return scm_number_to_string (SCM_MAKINUM(SCM_MICRO_VERSION),
+                               SCM_MAKINUM(10));
 }
 #undef FUNC_NAME
 
@@ -77,18 +92,33 @@ SCM_DEFINE (scm_minor_version, "minor-version", 0, 0, 0,
 
 SCM_DEFINE (scm_version, "version", 0, 0, 0, 
             (),
-	    "@deffnx primitive major-version\n"
-	    "@deffnx primitive minor-version\n"
-	    "Return a string describing Guile's version number, or its major or minor\n"
-	    "version numbers, respectively.\n\n"
-	    "@example\n"
-	    "(version) @result{} \"1.3a\"\n"
+	    "@deffnx {Scheme Procedure} major-version\n"
+	    "@deffnx {Scheme Procedure} minor-version\n"
+	    "@deffnx {Scheme Procedure} micro-version\n"
+	    "Return a string describing Guile's version number, or its major, minor\n"
+	    "or micro version number, respectively.\n\n"
+	    "@lisp\n"
+	    "(version) @result{} \"1.6.0\"\n"
 	    "(major-version) @result{} \"1\"\n"
-	    "(minor-version) @result{} \"3a\"\n"
-	    "@end example")
+	    "(minor-version) @result{} \"6\"\n"
+	    "(micro-version) @result{} \"0\"\n"
+	    "@end lisp")
 #define FUNC_NAME s_scm_version
 {
-  return scm_makfrom0str (GUILE_VERSION);
+
+  char version_str[3 * 4 + 3];
+
+#if SCM_MAJOR_VERSION > 9999 \
+    || SCM_MINOR_VERSION > 9999 \
+    || SCM_MICRO_VERSION > 9999
+# error version string may overflow buffer
+#endif
+  sprintf (version_str, "%d.%d.%d",
+           SCM_MAJOR_VERSION,
+           SCM_MINOR_VERSION,
+           SCM_MICRO_VERSION);
+
+  return scm_makfrom0str (version_str);
 }
 #undef FUNC_NAME
 
