@@ -2,7 +2,7 @@
 
 #ifndef VERSIONH
 #define VERSIONH
-/*	Copyright (C) 1995, 1996, 1998, 1999 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,9 +48,21 @@
 
 
 
-extern SCM scm_major_version SCM_P ((void));
-extern SCM scm_minor_version SCM_P ((void));
-extern SCM scm_version SCM_P ((void));
-extern void scm_init_version SCM_P ((void));
+#define SCM_MAJOR_VERSION 1
+#define SCM_MINOR_VERSION 6
+#define SCM_MICRO_VERSION 0
+
+extern SCM scm_major_version (void);
+extern SCM scm_minor_version (void);
+extern SCM scm_micro_version (void);
+extern SCM scm_version (void);
+extern void scm_init_version (void);
 
 #endif  /* VERSIONH */
+
+/*
+  Local Variables:
+  c-file-style: "gnu"
+  mode: c
+  End:
+*/
